/**
 *
 * Before running: please fill in Appid, APIKey, APISecret
 *
 # Voice Evaluation Streaming WebAPI Interface Call Example Interface Documentation (must see): https://global.xfyun.cn/doc/voiceservice/ise/API.html#interface-demos
 # The default evaluation example is the Chinese sentence type example, for other examples, please go to Demo to check the sample questions and audio examples and pay attention to modifying the evaluation parameter values, and go to the bottom of the platform document to download the audio sample questions as well.
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 * 
 */
const CryptoJS = require('crypto-js')
const WebSocket = require('ws')
const fs = require('fs')

// System Configuration
const config = {
    // Request Address
    hostUrl: "ws://ise-api-sg.xf-yun.com/v2/ise",
    host: "ise-api-sg.xf-yun.com",
    //Get it in Console 
    appid: "XXXXXXXX",
    //Get it in Console 
    apiSecret: "XXXXXXXXXXXXXXXXXXXXX",
    //Get it in Console
    apiKey: "XXXXXXXXXXXXXXXXXXXXX",
    file: "./read_sentence_en.pcm", //Please fill in the path of your audio file
    uri: "/v2/ise",
    highWaterMark: 1280,
}

// Frame Definition
const FRAME = {
    STATUS_FIRST_FRAME: 0,
    STATUS_CONTINUE_FRAME: 1,
    STATUS_LAST_FRAME: 2
}
// Read directly from the file
const text = '\uFEFF' + fs.readFileSync('./read_sentence_en.txt', 'utf-8');
// Get current time RFC1123 format
let date = (new Date().toUTCString())
// Set the current temporary state to initialized
let status = FRAME.STATUS_FIRST_FRAME

let wssUrl = config.hostUrl + "?authorization=" + getAuthStr(date) + "&date=" + date + "&host=" + config.host
let ws = new WebSocket(wssUrl)

// Connection established, read data for identification
ws.on('open', (event) => {
    console.log("websocket connect!")
    var readerStream = fs.createReadStream(config.file, {
        highWaterMark: config.highWaterMark
    });
    readerStream.on('data', function (chunk) {
        send(chunk)
        var stop = new Date().getTime();
        while (new Date().getTime() < stop + 40) {
            ;
        }
    });
    // End of final frame transmission
    readerStream.on('end', function () {
        status = FRAME.STATUS_LAST_FRAME
        send("")
    });
})

// Processing after getting the recognition result, for reference only, specific business specific treatment
ws.on('message', (data, err) => {
    if (err) {
        console.log(`err:${err}`)
        return
    }
    res = JSON.parse(data)
    if (res.code != 0) {
        console.log(`error code ${res.code}, reason ${res.message}`)
        return
    }

    if (res.data.status == 2) {
        const {data} = res.data
        console.log(res.data)
        let b = Buffer.from(data, 'base64')
        let iseResult = `final identification results: ${b.toString()}`
        console.log(iseResult)
    }
    // ... do something
})

// Resource release
ws.on('close', () => {
    console.log('connect close!')
})

// build error
ws.on('error', (err) => {
    console.log("websocket connect err: " + err)
})

// Authentication Signature
function getAuthStr(date) {
    let signatureOrigin = `host: ${config.host}\ndate: ${date}\nGET ${config.uri} HTTP/1.1`
    let signatureSha = CryptoJS.HmacSHA256(signatureOrigin, config.apiSecret)
    let signature = CryptoJS.enc.Base64.stringify(signatureSha)
    let authorizationOrigin = `api_key="${config.apiKey}", algorithm="hmac-sha256", headers="host date request-line", signature="${signature}"`
    let authStr = CryptoJS.enc.Base64.stringify(CryptoJS.enc.Utf8.parse(authorizationOrigin))
    return authStr
}

// transmission data
function send(data) {
    let frame = "";
    switch (status) {
        case FRAME.STATUS_FIRST_FRAME:
            // First data transmission:
            frame = {
                "common": {app_id: config.appid},
                "business": {
                    // 	Service type designation ise (open for review)
                    "sub": "ise",
                    // Chinese: cn_vip English: en_vip
                    "ent": "en_vip",
                    // Question type: Sentence reading
                    "category": "read_sentence",
                    // Text to be reviewed encoding utf-8
                    // "text": `\uFEFF${"今天天气怎么样"}`,
                    "text": text,
                    // text to be evaluated utf-8 gbk
                    "tte": "utf-8",
                    "rstcd": "utf8",
                    // skip ttp and use the text in ssb for evaluation (use it in conjunction with the cmd parameter),default true
                    "ttp_skip": true,
                    "cmd": "ssb",
                    "aue": "raw",
                    "auf": "audio/L16;rate=16000"
                },
                "data": {"status": 0}
            }
            ws.send(JSON.stringify(frame))
            // follow-up data sending
            frame = {
                "common": {"app_id": config.appid},
                "business": {"aus": 1, "cmd": "auw", "aue": "raw"},
                "data": {"status": 1, "data": data.toString('base64')}
            }
            status = FRAME.STATUS_CONTINUE_FRAME;
            break;
        case FRAME.STATUS_CONTINUE_FRAME:
            frame = {
                "common": {"app_id": config.appid},
                "business": {"aus": 2, "cmd": "auw", "aue": "raw"},
                "data": {"status": 1, "data": data.toString('base64')}
            }
            break;
        case FRAME.STATUS_LAST_FRAME:
            frame = {
                "common": {"app_id": config.appid},
                "business": {"aus": 4, "cmd": "auw", "aue": "raw"},
                "data": {"status": 2, "data": data.toString('base64')}
            }
            break;
    }
    ws.send(JSON.stringify(frame))
}